/*-
 * Copyright (c) 2005 Fredrik Lindberg. <fli at shapeshifter dot se>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * This is a primitive bitmap font engine
 * The fonts are stored as 8x8 matrix bitmaps and are compiled into
 * the program at compile time.
 */

#include <stdio.h>
#include <stdlib.h>
#if __FreeBSD_version >= 500000
# include <stdint.h>
#else
# include <inttypes.h>
#endif
#include <stdarg.h>
#include <string.h>

#include "mylibpng.h"
#include "font.h"
#include "font_terminal.h"

/*
 * Print text to image data
 * This routine is quite slow :( 
 */
void
mypng_printf(pngimage img, uint32_t x, uint32_t y, pngcolor8 *color, const char *fmt, ...)
{
	va_list ap;
	int cx, cy, i,j;
	char *buffer, *p;

	va_start(ap, fmt);
	vasprintf(&buffer, fmt, ap);
	va_end(ap);

	cx = x;

	/* Check if our string is too long, if so, shorten it */
	i = strlen(buffer);
	while ((i * 8) + cx > img->width && i >= 0) {
		buffer[i--] = '\0';
	}

	/* Step through the string and plot each character */
	for (p = buffer; *p != '\0'; p++) {
		fontmap_t *fmap;
		i = 0;
		cy = y;

		/* Lookup character */
		do {
			fmap = &terminal[i++];
			if (fmap->ch == *p)
				break;

		} while (fmap->ch != 0);


		/* Got a whitespace, increase x-offset */
		if (*p == ' ') {
			cx += 8;
		}

		/* We have a character, plot it */
		else if (fmap->ch != 0) {
			for (j = cy + 8; cy < j; cy++) {
				for (i = 0; i < 8; i++) {
					if (fmap->bitmap[cy-y] & 1 << i) {
						mypng_plot8_opacity(img, cx+i, cy, color);
					}
				}
			}
			cx += 7;
		}
	}

	free(buffer);
}
